@@ -102,9 +102,16 @@ update_color (GcalEventWidget *self)
   GQuark color_id;
   gchar *color_str;
   gchar *css_class;
+  GDateTime *now;
+  gboolean date_compare;
 
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
   color = gcal_event_get_color (self->event);
+  now = g_date_time_new_now_local ();
+  date_compare = g_date_time_compare (self->dt_end, now);
+
+  /* Fades out an event that's earlier than the current date */
+  gtk_widget_set_opacity (GTK_WIDGET (self), date_compare < 0 ? 0.6 : 1.0);
 
   /* Remove the old style class */
   if (self->css_class)
@@ -133,6 +140,7 @@ update_color (GcalEventWidget *self)
   /* Keep the current style around, so we can remove it later */
   self->css_class = css_class;
 
+  g_clear_pointer (&now, g_date_time_unref);
   g_free (color_str);
 }
 
@@ -478,7 +486,7 @@ gcal_event_widget_draw (GtkWidget *widget,
       g_clear_pointer (&start_time, g_free);
     }
 
-  /* FIXME for RTL alignment and icons positions */
+  /* Retrieve the font description */
   gtk_style_context_get (context, state, "font", &font_desc, NULL);
   layout = gtk_widget_create_pango_layout (widget, display_text);
   pango_layout_set_font_description (layout, font_desc);
