@@ -21,6 +21,7 @@
 #define __GCAL_MANAGER_H__
 
 #include "e-cal-data-model.h"
+#include "gcal-event.h"
 
 #include <libical/icaltime.h>
 #include <goa/goa.h>
@@ -28,41 +29,8 @@
 G_BEGIN_DECLS
 
 #define GCAL_TYPE_MANAGER                            (gcal_manager_get_type ())
-#define GCAL_MANAGER(obj)                            (G_TYPE_CHECK_INSTANCE_CAST((obj), GCAL_TYPE_MANAGER, GcalManager))
-#define GCAL_MANAGER_CLASS(klass)                    (G_TYPE_CHECK_CLASS_CAST((klass), GCAL_TYPE_MANAGER, GcalManagerClass))
-#define GCAL_IS_MANAGER(obj)                         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GCAL_TYPE_MANAGER))
-#define GCAL_IS_MANAGER_CLASS(klass)                 (G_TYPE_CHECK_CLASS_TYPE((klass), GCAL_TYPE_MANAGER))
-#define GCAL_MANAGER_GET_CLASS(obj)                  (G_TYPE_INSTANCE_GET_CLASS((obj), GCAL_TYPE_MANAGER, GcalManagerClass))
 
-typedef struct _GcalManager                           GcalManager;
-typedef struct _GcalManagerClass                      GcalManagerClass;
-
-struct _GcalManager
-{
-  GObject parent;
-};
-
-struct _GcalManagerClass
-{
-  GObjectClass parent_class;
-
-  /* signals */
-  void (*source_activated)  (GcalManager *manager, ESource *source, gboolean active);
-  void (*source_added)  (GcalManager *manager, ESource *source, gboolean enabled);
-  void (*source_removed)  (GcalManager *manager, ESource *source);
-  void (*source_enabled)  (GcalManager *manager, ESource *source, gboolean enabled);
-  void (*load_completed)  (GcalManager *manager);
-  void (*query_completed) (GcalManager *manager);
-  void (*goa_client_ready) (GcalManager *manager, GoaClient *client);
-};
-
-typedef struct
-{
-  ESource       *source;
-  ECalComponent *event_component;
-} GcalEventData;
-
-GType          gcal_manager_get_type                (void);
+G_DECLARE_FINAL_TYPE (GcalManager, gcal_manager, GCAL, MANAGER, GObject)
 
 GcalManager*   gcal_manager_new_with_settings       (GSettings          *settings);
 
@@ -136,23 +104,19 @@ gboolean       gcal_manager_is_client_writable      (GcalManager        *manager
 
 /* Create method */
 void           gcal_manager_create_event            (GcalManager        *manager,
-                                                     ESource            *source,
-                                                     ECalComponent      *component);
+                                                     GcalEvent          *event);
 
 /* Update method */
 void           gcal_manager_update_event            (GcalManager        *manager,
-                                                     ESource            *source,
-                                                     ECalComponent      *component);
+                                                     GcalEvent          *event);
 
 /* Remove method */
 void           gcal_manager_remove_event            (GcalManager        *manager,
-                                                     ESource            *source,
-                                                     ECalComponent      *component);
+                                                     GcalEvent          *event);
 
 /* Set methods */
 void           gcal_manager_move_event_to_source    (GcalManager        *manager,
-                                                     ECalComponent      *component,
-                                                     ESource            *source,
+                                                     GcalEvent          *event,
                                                      ESource            *dest);
 
 GList*         gcal_manager_get_events              (GcalManager        *manager,
@@ -161,7 +125,7 @@ GList*         gcal_manager_get_events              (GcalManager        *manager
 
 gboolean       gcal_manager_load_completed          (GcalManager        *manager);
 
-GcalEventData* gcal_manager_get_event_from_shell_search (GcalManager        *manager,
+GcalEvent*     gcal_manager_get_event_from_shell_search (GcalManager        *manager,
                                                          const gchar        *uuid);
 
 gboolean       gcal_manager_is_goa_client_ready    (GcalManager        *manager);
