@@ -27,9 +27,10 @@
 
 #define NAVIGATOR_CELL_WIDTH (210 + 15)
 #define NAVIGATOR_CELL_HEIGHT 210
+#define NAVIGATOR_HEADER_HEIGHT 52
 #define SIDEBAR_PREFERRED_WIDTH 200
 #define VISUAL_CLUES_SIDE 3.0
-#define WEEK_NUMBER_PADDING 3.0
+#define WEEK_NUMBER_MARGIN 3.0
 
 typedef struct
 {
@@ -67,6 +68,9 @@ struct _GcalYearView
 
   /* geometry info */
   GridData     *navigator_grid;
+  guint         number_of_columns;
+  guint         number_of_rows;
+
 
   /* state flags */
   gboolean      popover_mode;
@@ -87,6 +91,10 @@ struct _GcalYearView
    */
   gboolean      use_24h_format;
 
+  /* show week numbers from GNOME Shell settings */
+  GSettings    *shell_settings;
+  gboolean      show_week_numbers;
+
   /* text direction factors */
   gint          k;
 
@@ -100,6 +108,7 @@ struct _GcalYearView
 enum {
   PROP_0,
   PROP_DATE,
+  PROP_SHOW_WEEK_NUMBERS,
   LAST_PROP
 };
 
@@ -112,7 +121,7 @@ enum
 static guint signals[NUM_SIGNALS] = { 0, };
 
 static void   reset_sidebar (GcalYearView *year_view);
-static void   gcal_view_interface_init (GcalViewIface *iface);
+static void   gcal_view_interface_init (GcalViewInterface *iface);
 static void   gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GcalYearView, gcal_year_view, GTK_TYPE_BOX,
@@ -285,11 +294,7 @@ add_event_to_day_array (GcalYearView  *year_view,
 
   /* normalize date on each new event */
   date = icaltime_to_datetime (year_view->start_selected_date);
-  second_date = g_date_time_new (gcal_event_get_timezone (event),
-                                 g_date_time_get_year (date),
-                                 g_date_time_get_month (date),
-                                 g_date_time_get_day_of_month (date) + 1,
-                                 0, 0, 0);
+  second_date = g_date_time_add_days (date, 1);
 
   /* marking and cloning */
   for (i = 0; i < days_span; i++)
@@ -546,7 +551,7 @@ calculate_day_month_for_coord (GcalYearView *year_view,
                                gint         *out_day,
                                gint         *out_month)
 {
-  gint row, column, i, sw, clicked_cell, day, month;
+  gint row, column, i, sw, clicked_cell, day, month, columns_or_rows;
   gdouble box_side;
 
   row = -1;
@@ -554,18 +559,19 @@ calculate_day_month_for_coord (GcalYearView *year_view,
   box_side = year_view->navigator_grid->box_side;
   sw = 1 - 2 * year_view->k;
 
-  /* y selection */
-  for (i = 0; i < 4 && ((row == -1) || (column == -1)); i++)
+  columns_or_rows = year_view->number_of_columns > year_view->number_of_rows ? year_view->number_of_columns : year_view->number_of_rows;
+
+  for (i = 0; i < columns_or_rows && ((row == -1) || (column == -1)); i++)
     {
       if (row == -1 &&
-          y > year_view->navigator_grid->coordinates[i * 4].y + box_side &&
-          y < year_view->navigator_grid->coordinates[i * 4].y + box_side * 7)
+          y > year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y &&
+          y < year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y + box_side * 7)
         {
           row = i;
         }
       if (column == -1 &&
-          x > year_view->navigator_grid->coordinates[i].x + box_side * 0.5 &&
-          x < year_view->navigator_grid->coordinates[i].x + box_side * 7.5)
+          x > year_view->navigator_grid->coordinates[i].x + box_side * (1 - year_view->k) &&
+          x < year_view->navigator_grid->coordinates[i].x + box_side * (8 - year_view->k))
         {
           column = i;
         }
@@ -574,9 +580,9 @@ calculate_day_month_for_coord (GcalYearView *year_view,
   if (row == -1 || column == -1)
     return FALSE;
 
-  month = row * 4 + column;
+  month = row * year_view->number_of_columns + column;
   row = (y - (year_view->navigator_grid->coordinates[month].y + box_side)) / box_side;
-  column = (x - (year_view->navigator_grid->coordinates[month].x + box_side * 0.5)) / box_side;
+  column = (x - (year_view->navigator_grid->coordinates[month].x + box_side * (1 - year_view->k))) / box_side;
   clicked_cell = row * 7 + column;
   day = 7 * ((clicked_cell + 7 * year_view->k) / 7) + sw * (clicked_cell % 7) + (1 - year_view->k);
   day -= ((time_day_of_week (1, month, year_view->date->year) - year_view->first_weekday + 7) % 7);
@@ -657,7 +663,7 @@ draw_month_grid (GcalYearView *year_view,
   gtk_style_context_get_color (context, state_flags, &color);
   cairo_set_line_width (cr, 0.2);
   gdk_cairo_set_source_rgba (cr, &color);
-  cairo_move_to (cr, x + box_side / 2, y + box_side + 0.4);
+  cairo_move_to (cr, x + box_side * (1 - year_view->k), y + box_side + 0.4);
   cairo_rel_line_to (cr, 7.0 * box_side, 0);
   cairo_stroke (cr);
 
@@ -673,7 +679,7 @@ draw_month_grid (GcalYearView *year_view,
   days_delay = (time_day_of_week (1, month_nr, year_view->date->year) - year_view->first_weekday + 7) % 7;
   days = days_delay + icaltime_days_in_month (month_nr + 1, year_view->date->year);
   shown_rows = ceil (days / 7.0);
-  sunday_idx = year_view->k * 6 + sw * ((7 - year_view->first_weekday) % 7);
+  sunday_idx = year_view->k * 6 + sw * ((7 - year_view->first_weekday) % 7) + 1;
 
   start_date.day    = 1;
   start_date.month  = month_nr + 1;
@@ -690,7 +696,7 @@ draw_month_grid (GcalYearView *year_view,
 
   for (i = 0; i < 7 * shown_rows; i++)
     {
-      column = i % 7;
+      column = (i % 7) + 1;
       row = i / 7;
 
       j = 7 * ((i + 7 * year_view->k) / 7) + sw * (i % 7) + (1 - year_view->k);
@@ -747,11 +753,11 @@ draw_month_grid (GcalYearView *year_view,
 
           /* FIXME: hardcoded padding of the number background */
           gtk_render_background (context, cr,
-                                 box_side * (column + 0.5 + year_view->k) + x + sw * box_padding_start - year_view->k * layout_width - 2.0,
+                                 box_side * column + x + sw * box_padding_start - year_view->k * layout_width - 2.0,
                                  box_side * (row + 1) + y + box_padding_top - 1.0,
                                  layout_width + 4.0, layout_height + 2.0);
           gtk_render_layout (context, cr,
-                             box_side * (column + 0.5 + year_view->k) + x + sw * box_padding_start - year_view->k * layout_width,
+                             box_side * column + x + sw * box_padding_start - year_view->k * layout_width,
                              box_side * (row + 1) + y + box_padding_top,
                              clayout);
 
@@ -769,7 +775,7 @@ draw_month_grid (GcalYearView *year_view,
           box_padding_start = (box_side - layout_width) / 2 > 0 ? (box_side - layout_width) / 2 : 0;
 
           gtk_render_layout (context, cr,
-                             box_side * (column + 0.5 + year_view->k) + x + sw * box_padding_start - year_view->k * layout_width,
+                             box_side * column + x + sw * box_padding_start - year_view->k * layout_width,
                              box_side * (row + 1) + y + box_padding_top,
                              slayout);
 
@@ -780,7 +786,7 @@ draw_month_grid (GcalYearView *year_view,
           gtk_style_context_save (context);
           gtk_style_context_add_class (context, "sunday");
           gtk_render_layout (context, cr,
-                             box_side * (column + 0.5 + year_view->k) + x + sw * box_padding_start - year_view->k * layout_width,
+                             box_side * column + x + sw * box_padding_start - year_view->k * layout_width,
                              box_side * (row + 1) + y + box_padding_top,
                              layout);
           gtk_style_context_restore (context);
@@ -788,7 +794,7 @@ draw_month_grid (GcalYearView *year_view,
       else
         {
           gtk_render_layout (context, cr,
-                             box_side * (column + 0.5 + year_view->k) + x + sw * box_padding_start - year_view->k * layout_width,
+                             box_side * column + x + sw * box_padding_start - year_view->k * layout_width,
                              box_side * (row + 1) + y + box_padding_top,
                              layout);
         }
@@ -805,7 +811,7 @@ draw_month_grid (GcalYearView *year_view,
             gtk_style_context_add_class (context, "with-events-sunday");
           box_padding_start = (box_side - VISUAL_CLUES_SIDE) / 2 > 0 ? (box_side - VISUAL_CLUES_SIDE) / 2 : 0;
           gtk_render_background (context, cr,
-                                 box_side * (column + 0.5 + year_view->k) + x + sw * box_padding_start - year_view->k * VISUAL_CLUES_SIDE,
+                                 box_side * column + x + sw * box_padding_start - year_view->k * VISUAL_CLUES_SIDE,
                                  box_side * (row + 1) + y + box_padding_top + layout_height + 2.0,
                                  VISUAL_CLUES_SIDE, VISUAL_CLUES_SIDE);
           gtk_style_context_restore (context);
@@ -840,24 +846,27 @@ draw_month_grid (GcalYearView *year_view,
       else
         *weeks_counter = *weeks_counter + 1;
 
-      nr_week = g_strdup_printf ("%d", *weeks_counter);
+      if (year_view->show_week_numbers)
+        {
+          nr_week = g_strdup_printf ("%d", *weeks_counter);
 
-      pango_layout_set_text (layout, nr_week, -1);
-      pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
-      box_padding_top = (box_side - layout_height) / 2.0 > 0 ? (box_side - layout_height) / 2.0 : 0;
-      box_padding_start = (box_side - layout_width) / 2.0 > 0 ? (box_side - layout_width) / 2.0 : 0;
+          pango_layout_set_text (layout, nr_week, -1);
+          pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
+          box_padding_top = (box_side - layout_height) / 2.0 > 0 ? (box_side - layout_height) / 2.0 : 0;
+          box_padding_start = (box_side - layout_width) / 2.0 > 0 ? (box_side - layout_width) / 2.0 : 0;
 
-      gtk_render_background (context, cr,
-                             box_side * (- 0.5) + x + sw * WEEK_NUMBER_PADDING + year_view->k * (8 * box_side + WEEK_NUMBER_PADDING * 2),
-                             box_side * (i + 1) + y + WEEK_NUMBER_PADDING,
-                             box_side - WEEK_NUMBER_PADDING * 2, box_side - WEEK_NUMBER_PADDING * 2);
+          gtk_render_background (context, cr,
+                                 x + sw * box_padding_top + year_view->k * (8 * box_side - layout_height) - WEEK_NUMBER_MARGIN,
+                                 box_side * (i + 1) + y + box_padding_top - WEEK_NUMBER_MARGIN,
+                                 layout_height + WEEK_NUMBER_MARGIN * 2, layout_height + WEEK_NUMBER_MARGIN * 2);
 
-      gtk_render_layout (context, cr,
-                         box_side * (- 0.5) + x + sw * box_padding_start + year_view->k * (9 * box_side - layout_width),
-                         box_side * (i + 1) + y + box_padding_top,
-                         layout);
+          gtk_render_layout (context, cr,
+                             x + sw * box_padding_start + year_view->k * (8 * box_side - layout_width),
+                             box_side * (i + 1) + y + box_padding_top,
+                             layout);
 
-      g_free (nr_week);
+          g_free (nr_week);
+        }
     }
   gtk_style_context_restore (context);
 
@@ -921,23 +930,23 @@ draw_navigator (GcalYearView *year_view,
   header_height = header_padding_top * 2 + layout_height;
   height = gtk_widget_get_allocated_height (widget) - header_height;
 
-  if (((width / 4.0) / 8.0) < ((height / 3.0) / 7.0))
-    box_side = (width / 4.0) / 8.0;
-  else
-    box_side = (height / 3.0) / 7.0;
+  box_side = NAVIGATOR_CELL_WIDTH / 8.0;
 
-  real_padding_left = (width - (8 * 4 * box_side)) / 5.0;
-  real_padding_top = (height - (7 * 3 * box_side)) / 4.0;
+  real_padding_left = (width - NAVIGATOR_CELL_WIDTH * year_view->number_of_columns) / (year_view->number_of_columns + 1);
+  real_padding_top = (height - NAVIGATOR_CELL_HEIGHT * year_view->number_of_rows) / (year_view->number_of_rows + 1);
+
+  if (real_padding_top < 0)
+    real_padding_top = 0;
 
   year_view->navigator_grid->box_side = box_side;
   weeks_counter = year_view->first_week_of_year;
   for (i = 0; i < 12; i++)
     {
-      gint row = i / 4;
-      gint column = year_view->k * 3 + sw * (i % 4);
+      gint row = i / year_view->number_of_columns;
+      gint column = year_view->k * (year_view->number_of_columns - 1) + sw * (i % year_view->number_of_columns);
 
-      year_view->navigator_grid->coordinates[i].x = (column + 1) * real_padding_left + column * box_side * 8;
-      year_view->navigator_grid->coordinates[i].y = (row + 1) * real_padding_top + row * box_side * 7 + header_height;
+      year_view->navigator_grid->coordinates[i].x = (column + 1) * real_padding_left + column * NAVIGATOR_CELL_WIDTH;
+      year_view->navigator_grid->coordinates[i].y = row * real_padding_top + row * NAVIGATOR_CELL_HEIGHT + header_height;
       draw_month_grid (year_view, widget, cr, i, &weeks_counter);
     }
 
@@ -1054,12 +1063,16 @@ add_event_clicked_cb (GcalYearView *year_view,
   else
     {
       icaltimetype *dtstart, *dtend;
+      GDateTime *tmp_date;
 
       dtstart = year_view->start_selected_date;
       dtend = year_view->end_selected_date;
 
       start_date = g_date_time_new_local (dtstart->year, dtstart->month, dtstart->day, 0, 0, 0);
-      end_date = g_date_time_new_local (dtend->year, dtend->month, dtend->day + 1, 0, 0, 0);
+      tmp_date = g_date_time_new_local (dtend->year, dtend->month, dtend->day, 0, 0, 0);
+      end_date = g_date_time_add_days (tmp_date, 1);
+
+      g_clear_pointer (&tmp_date, g_date_time_unref);
     }
 
   if (year_view->popover_mode)
@@ -1091,6 +1104,8 @@ gcal_year_view_finalize (GObject *object)
 
   g_clear_pointer (&year_view->date, g_free);
 
+  g_clear_object (&year_view->shell_settings);
+
   G_OBJECT_CLASS (gcal_year_view_parent_class)->finalize (object);
 }
 
@@ -1108,6 +1123,10 @@ gcal_year_view_get_property (GObject    *object,
       g_value_set_boxed (value, self->date);
       break;
 
+    case PROP_SHOW_WEEK_NUMBERS:
+      g_value_set_boolean (value, self->show_week_numbers);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -1119,10 +1138,20 @@ gcal_year_view_set_property (GObject      *object,
                              const GValue *value,
                              GParamSpec   *pspec)
 {
+  GcalYearView *self = GCAL_YEAR_VIEW (object);
+
   switch (prop_id)
     {
     case PROP_DATE:
-      update_date (GCAL_YEAR_VIEW (object), g_value_dup_boxed (value));
+      update_date (self, g_value_dup_boxed (value));
+      break;
+
+    case PROP_SHOW_WEEK_NUMBERS:
+      if (self->show_week_numbers != g_value_get_boolean (value))
+        {
+          self->show_week_numbers = g_value_get_boolean (value);
+          g_object_notify (object, "show-week-numbers");
+        }
       break;
 
     default:
@@ -1137,40 +1166,20 @@ gcal_year_view_get_preferred_width (GtkWidget *widget,
 {
   GcalYearView *year_view = GCAL_YEAR_VIEW (widget);
   GtkStyleContext *context;
-  gint padding_left;
+  gint padding_left, padding_right, hpadding;
 
   context = gtk_widget_get_style_context (year_view->navigator);
 
   gtk_style_context_get (context,
                          gtk_style_context_get_state (context),
-                         "padding-left", &padding_left, NULL);
+                         "padding-left", &padding_left,
+                         "padding-right", &padding_right, NULL);
 
+  hpadding = padding_left + padding_right;
   if (minimum != NULL)
-    *minimum = NAVIGATOR_CELL_WIDTH * 4 + padding_left * 8;
+    *minimum = hpadding + NAVIGATOR_CELL_WIDTH;
   if (natural != NULL)
-    *natural = NAVIGATOR_CELL_WIDTH * 4 + padding_left * 8 + SIDEBAR_PREFERRED_WIDTH;
-}
-
-static void
-gcal_year_view_get_preferred_height_for_width (GtkWidget *widget,
-                                               gint       width,
-                                               gint      *minimum,
-                                               gint      *natural)
-{
-  GcalYearView *year_view = GCAL_YEAR_VIEW (widget);
-  GtkStyleContext *context;
-  gint padding_top;
-
-  context = gtk_widget_get_style_context (year_view->navigator);
-
-  gtk_style_context_get (context,
-                         gtk_style_context_get_state (context),
-                         "padding-top", &padding_top, NULL);
-
-  if (minimum != NULL)
-    *minimum = NAVIGATOR_CELL_HEIGHT * 3 + padding_top * 6;
-  if (natural != NULL)
-    *natural = NAVIGATOR_CELL_HEIGHT * 3 + padding_top * 6;
+    *natural = hpadding + NAVIGATOR_CELL_WIDTH + SIDEBAR_PREFERRED_WIDTH;
 }
 
 static void
@@ -1179,15 +1188,29 @@ gcal_year_view_size_allocate (GtkWidget     *widget,
 {
   GcalYearView *year_view = GCAL_YEAR_VIEW (widget);
   GtkStyleContext *context;
-  gint padding_left;
+  gint padding_left, padding_right, padding_top, padding_bottom, hpadding, vpadding;
 
   context = gtk_widget_get_style_context (widget);
   gtk_style_context_save (context);
   gtk_style_context_add_class (context, "year-navigator");
-  gtk_style_context_get (context, gtk_style_context_get_state (context), "padding-left", &padding_left, NULL);
+  gtk_style_context_get (context,
+                         gtk_style_context_get_state (context),
+                         "padding-left", &padding_left,
+                         "padding-right", &padding_right,
+                         "padding-top", &padding_top,
+                         "padding-bottom", &padding_bottom, NULL);
   gtk_style_context_restore (context);
 
-  year_view->popover_mode = (alloc->width < NAVIGATOR_CELL_WIDTH * 4 + padding_left * 8 + SIDEBAR_PREFERRED_WIDTH);
+  year_view->number_of_columns = (alloc->width - SIDEBAR_PREFERRED_WIDTH) / NAVIGATOR_CELL_WIDTH;
+  year_view->number_of_rows = ceil (12.0 / year_view->number_of_columns);
+
+  hpadding = padding_left + padding_right;
+  vpadding = padding_top + padding_bottom;
+  gtk_widget_set_size_request (year_view->navigator,
+                               hpadding + NAVIGATOR_CELL_WIDTH * year_view->number_of_columns,
+                               vpadding + NAVIGATOR_CELL_HEIGHT * year_view->number_of_rows + NAVIGATOR_HEADER_HEIGHT);
+
+  year_view->popover_mode = (alloc->width < padding_left + NAVIGATOR_CELL_WIDTH * year_view->number_of_columns + padding_right + SIDEBAR_PREFERRED_WIDTH);
   if (year_view->popover_mode && !gtk_widget_is_ancestor (year_view->events_sidebar, year_view->popover))
     {
       g_object_ref (year_view->sidebar);
@@ -1419,12 +1442,19 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
   object_class->set_property = gcal_year_view_set_property;
 
   widget_class->get_preferred_width = gcal_year_view_get_preferred_width;
-  widget_class->get_preferred_height_for_width = gcal_year_view_get_preferred_height_for_width;
   widget_class->size_allocate = gcal_year_view_size_allocate;
   widget_class->direction_changed = gcal_year_view_direction_changed;
 
   g_object_class_override_property (object_class, PROP_DATE, "active-date");
 
+  g_object_class_install_property (object_class,
+                                   PROP_SHOW_WEEK_NUMBERS,
+                                   g_param_spec_boolean ("show-week-numbers",
+                                                         "Show Week Numbers",
+                                                         "Show Week Numbers Column",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
+
   /* FIXME: it will problably go back to GcalView */
   signals[EVENT_ACTIVATED] = g_signal_new ("event-activated", GCAL_TYPE_YEAR_VIEW, G_SIGNAL_RUN_LAST,
                                            0,
@@ -1469,12 +1499,25 @@ gcal_year_view_init (GcalYearView *self)
   self->end_selected_date = g_new0 (icaltimetype, 1);
   self->end_selected_date->zone = e_cal_util_get_system_timezone ();
 
+  /* FIXME: this setting should be moved to gsettings-desktop-schemas */
+  if (g_strcmp0 (g_getenv ("XDG_CURRENT_DESKTOP"), "GNOME") == 0)
+    {
+      /* bind GNOME Shell' show week numbers property to GNOME Calendar's one */
+      self->shell_settings = g_settings_new ("org.gnome.shell.calendar");
+      g_settings_bind (self->shell_settings, "show-weekdate", self, "show-week-numbers", G_SETTINGS_BIND_DEFAULT);
+      g_signal_connect_swapped (self->shell_settings, "changed::show-weekdate", G_CALLBACK (gtk_widget_queue_allocate), self);
+    }
+
+  /* default number_of_columns and number_of_rows */
+  self->number_of_columns = 4;
+  self->number_of_rows = 3;
+
   gtk_list_box_set_header_func (GTK_LIST_BOX (self->events_sidebar), update_sidebar_headers, self, NULL);
   gtk_list_box_set_sort_func (GTK_LIST_BOX (self->events_sidebar), sidebar_sort_func, NULL, NULL);
 }
 
 static void
-gcal_view_interface_init (GcalViewIface *iface)
+gcal_view_interface_init (GcalViewInterface *iface)
 {
   /* FIXME: implement what's needed */
   iface->get_children_by_uuid = gcal_year_view_get_children_by_uuid;
